// SPDX-License-Identifier: Apache-2.0
// Copyright (C) 2025 Advanced Micro Devices, Inc. All rights reserved.

#pragma once

#include <string>

namespace xrt_smi_default {

std::string 
get_default_smi_config();

} // namespace xrt_smi_default
