// SPDX-License-Identifier: Apache-2.0
// Copyright (C) 2025 Advanced Micro Devices, Inc. All rights reserved.

#include <string>
#include "core/common/smi.h"

namespace shim_pcie::smi {

  std::string 
  get_smi_config();

} // namespace shim_pcie::smi
